<?php
namespace Ig\IgRuckzuckevent\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use Ig\IgRuckzuckevent\Domain\Model\Organizer;
use Internetgalerie\IgDynval\Controller\DynamicValidationActionController;

class OrganizerController extends ActionController
{
    use DynamicValidationActionController;

    /**
     * The repository for organizers
     *
     * @var \Ig\IgRuckzuckevent\Domain\Repository\OrganizerRepository
     */
    protected $organizerRepository = null;

    /**
     * @param \Ig\IgRuckzuckevent\Domain\Repository\OrganizerRepository $organizerRepository
     */
    public function injectOrganizerRepository(\Ig\IgRuckzuckevent\Domain\Repository\OrganizerRepository $organizerRepository)
    {
        $this->organizerRepository = $organizerRepository;
    }

    public function initializeView($view)
    {
        $this->view->assign('controller', 'Organizer');
        $this->view->assign('pageUid', $GLOBALS['TSFE']->id);
    }

    /**
     * Liast all events with possibility to sort and filter them
     *
     * @return void
     */
    public function listAction()
    {
        $organizers = $this->organizerRepository->findAll();
        $this->view->assign('organizers', $organizers);
    }

    /**
     * Liast all events with possibility to sort and filter them
     *
     * @return void
     */
    public function organizerListAction()
    {
        $organizers = $this->organizerRepository->findAll();
        $this->view->assign('organizers', $organizers);
    }

    /**
     * New action
     *
     * @return void
     */
    public function newAction()
    {
    }

    /**
     * Create action
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Organizer $newOrganizer
     * @TYPO3\CMS\Extbase\Annotation\Validate("Internetgalerie\IgDynval\Validation\Validator\DynamicValidator", param="newOrganizer")
     * @return void
     */
    public function createAction(Organizer $newOrganizer)
    {
        $this->organizerRepository->add($newOrganizer);
        $this->redirect('list');
    }

    /**
     * Edit action
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Organizer $organizer
     * @return void
     * @TYPO3\CMS\Extbase\Annotation\IgnoreValidation $organizer
     */
    public function editAction(Organizer $organizer)
    {
        $this->view->assign('organizer', $organizer);
    }

    /**
     * Update action
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Organizer $organizer
     * @TYPO3\CMS\Extbase\Annotation\Validate("Internetgalerie\IgDynval\Validation\Validator\DynamicValidator", param="organizer")
     * @return void
     */
    public function updateAction(Organizer $organizer)
    {
        $this->organizerRepository->update($organizer);
        $this->redirect('list');
    }

    /**
     * Delete action
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Organizer $organizer
     * @return void
     */
    public function deleteAction(Organizer $organizer)
    {
        $this->organizerRepository->remove($organizer);
        $this->redirect('list');
    }

}
